/* This program is Copyright (c) 1990 David Allen.  It may be freely
   distributed as long as you leave my name and copyright notice on it.
   I'd really like your comments and feedback; send e-mail to
   davea@vlsi.ll.mit.edu, or send us-mail to David Allen, 10 O'Moore Ave,
   Maynard, MA 01754. */


/* This file contains only functions which are specific to the Amiga.  It
   connects to the other source files via a small number of functions.
   Main() is here.  It calls init() and onestep().  The other source files
   can call rnd(), draw() and panic() from this file; these are at the end
   of the file.  My purpose here is to make tec?.c machine
   independent, with all the machine dependencies in this file. */


#include "const.h"
#include "var.h"
#include "exec/types.h"
#include "intuition/intuition.h"
#include "intuition/intuitionbase.h"

extern short step; /* Defined in tec1.c */
extern unsigned char t[2][MAXX][MAXY]; /* Defined in tec1.c */
extern unsigned long RangeSeed; /* Amiga random number seed */
extern struct IntuitionBase *IntuitionBase;

struct GfxBase        *GfxBase;
struct RastPort       *rp;
struct ViewPort       *vp;
struct Window         *w = NULL;
struct Screen         *screen;
struct IntuiMessage   *message;
unsigned long         class;
unsigned short        code;

/* 32 colors - 0-3 are used by Intuition; black background, grey gadgets, */
/* white shadows, red highlighting.  Then there are 5 shades of blue from */
/* dark to light, 5 of green, 5 of red, 5 of purple, 5 white, and 3 yellow. */
/* A 5-bitplane screen is required. */
unsigned short colors [32] = {
   0x000, 0x888, 0xfff, 0x00f,
   0x007, 0x009, 0x00b, 0x00d, 0x00f,
   0x070, 0x090, 0x0b0, 0x0d0, 0x0f0,
   0x700, 0x900, 0xb00, 0xd00, 0xf00,
   0x707, 0x909, 0xb0b, 0xd0d, 0xf0f,
   0x777, 0x999, 0xbbb, 0xddd, 0xfff,
   0x099, 0x0bb, 0x0dd };

/* Structures for a custom lores screen of 5 bitplanes with a 320x200 */
/* window; use left-amiga-n and left-amiga-m to get back to workbench screen */

struct NewScreen ns = { 0L, 0L, 320L, 200L, 5L, 0, 1,
   NULL, CUSTOMSCREEN, NULL, (UBYTE *)0, NULL, NULL };

struct NewWindow nw = {
   0, 0, 320, 200, 0, 1, 0, 0, NULL, NULL,
   (UBYTE *) NULL, NULL, NULL,
   0, 0, 320, 200, CUSTOMSCREEN };


main (argc, argv) int argc; char **argv; {
   unsigned long t[3];

   /* Initialize random number generator using milliseconds of datestamp */
   DateStamp (t);  RangeSeed = t[2];

   /* Initialize everything */
   grafinit ();
   init (*++argv);

   /* Call onestep () once per step; check for mouse clicks */
   for (step=0; step<MAXSTEP; step++) {
      checkmouse ();
      onestep (); }

   /* Loop forever until user clicks close gadget - busy waiting, sorry! */
   while (1 == 1) checkmouse (); }


grafinit () {
   /* This is a standard kind of setup function; open all the libraries, then
   open the screen, then open the window.  If any of the calls fail, free the
   memory and return 0.  The rastport variable is kept for the drawing
   function and the colors are loaded. */

   GfxBase = (struct GfxBase *)OpenLibrary("graphics.library", 0L);
   IntuitionBase = (struct IntuitionBase *)OpenLibrary("intuition.library", 0L);
   screen = (struct Screen *) OpenScreen (&ns);
   if (screen == NULL) goto c2;
   vp = &screen->ViewPort; nw.Screen = screen;
   nw.Title = "Tectonics";
   nw.Flags = SIMPLE_REFRESH | WINDOWCLOSE;
   nw.IDCMPFlags = MOUSEBUTTONS | CLOSEWINDOW;
   w = (struct Window *) OpenWindow (&nw);
   if (w == NULL) goto c1;

   rp = w->RPort; SetDrMd (rp, 0);
   LoadRGB4 (vp, colors, 32);
   return (1);

   c1: if (w) CloseWindow (w);
   c2: CloseScreen (screen);
   CloseLibrary (IntuitionBase);
   CloseLibrary (GfxBase);
   return (0); }


grafexit () {
   /* Just close all the things that were opened, then exit(). */
   CloseWindow (w);
   CloseScreen (screen);
   CloseLibrary (IntuitionBase);
   CloseLibrary (GfxBase);
   exit (0); }


checkmouse () {
   /* Standard event handler; loop through all the messages that have come in,
   figure out whether they are clicks or gadget hits, and reply to them.
   Outside the loop, act on the clicks.  If the CLOSEWINDOW gadget was hit,
   exit the program; if the left button was clicked anywhere, print out the
   current values of t[src] using tecst () in tec1.c. */

   short mousecode = 0, i, j, k;

   while ((message=(struct IntuiMessage *)GetMsg(w->UserPort))!=NULL) {
      class = message->Class; code = message->Code;
      ReplyMsg (message);
      if (code == SELECTDOWN) mousecode = 1;
      if (class == CLOSEWINDOW) mousecode = -1; }
   if (mousecode == -1) grafexit ();
   if (mousecode == 1) tecst (step % 2, DRAWMODE); }


rnd (top) short top; { return (RangeRand (top)); }
   /* RangeRand is located in Amiga.lib */


panic (s) char *s; { printf ("PANIC: %s\n", s); grafexit (0); }
   /* Used when some fatal inconsistency is found in the database;
   its function is to immediately free all graphics memory and exit. */


draw (src) short src; {
   /* Takes the array m[src] and draws it on the screen, in a hopefully
   efficient way.  The function tries to make long horizontal
   patches that are all the same color.  Then they can be rendered by a
   graphics function that draws arbitrary rectangles quickly. */

   register short i, j, k, x;

   /* Erase the screen first by drawing a big black box */
   SetAPen (rp, 0); RectFill (rp, 14, 10, 280, 189); 

   /* For each scan line, start at the left edge */
   for (j=0, i=0; j<YSIZE; j++, i=0) {

      /* If the current square is not ocean, set x to its color */
      top: if ((x = t[src][i][j] >> 2) > 1) {

         /* Go as far along to the right as you can in this color */
         k = i+1; while (((t[src][k][j] >> 2) == x) && (k < XSIZE-1)) k++;

         /* Draw a short, wide rectangle */
         if (x > 27) x = 27; SetAPen (rp, x+4);
         RectFill (rp, 3*i + 14, 2*j + 10, 3*k + 13, 2*j + 11);
         i = k-1; }

      /* If not at end of scanline, do more; else start next scanline */
      if (i < XSIZE-1) { i++; goto top; } } }
